@@ -1,21 +1,17 @@
 import asyncio
-import functools
 import concurrent.futures
-import time
+import functools
 import random
-import logging
-
-from enum import IntEnum
+import time
 from datetime import datetime
+from enum import IntEnum
 
-from bluepy.btle import BTLEException
-from . import eq3btsmart as eq3
-from .connection import BTLEConnection
 from dbus import DBusException
 
-from .. import utils
-from .. import fhem
+from .. import fhem, utils
 from ..generic import FhemModule
+from . import eq3btsmart as eq3
+from .connection import BTLEConnection
 
 
 class Mode(IntEnum):
@@ -69,6 +65,31 @@ class eq3bt(FhemModule):
                 "args": ["cons_var"],
                 "options": "all,consumption,consumptionToday,consumptionYesterday",
             },
+            "temperatureOffset": {
+                "args": ["offset"],
+                "params": {"offset": {"format": "float"}},
+                "options": "slider,-3.5,0.5,3.5,1",
+            },
+            "windowOpenTemperature": {
+                "args": ["temp"],
+                "params": {"temp": {"format": "float"}},
+                "options": "slider,4.5,0.5,30,1",
+            },
+            "windowOpenTime": {
+                "args": ["seconds"],
+                "params": {"seconds": {"format": "int"}},
+                "options": "slider,0,1,3600,1",
+            },
+            "ecoTemperature": {
+                "args": ["temp"],
+                "params": {"temp": {"format": "float"}},
+                "options": "slider,4.5,0.5,30,1",
+            },
+            "comfortTemperature": {
+                "args": ["temp"],
+                "params": {"temp": {"format": "float"}},
+                "options": "slider,4.5,0.5,30,1",
+            },
         }
         self.set_set_config(set_list_conf)
 
@@ -79,6 +100,8 @@ class eq3bt(FhemModule):
     async def Define(self, hash, args, argsh):
         await super().Define(hash, args, argsh)
         self.hash = hash
+        if len(args) < 4:
+            return "Usage: define eq3_livingroom PythonModule eq3bt <MAC>"
         self._mac = args[3]
         self.hash["MAC"] = self._mac
         self.logger.info(f"Define: eq3bt {self._mac}")
@@ -173,7 +196,7 @@ class eq3bt(FhemModule):
                         self.hash, "state", "update", 1
                     )
                 await self.update_all()
-            except:
+            except Exception:
                 self.logger.error(f"Failed to update, retry in {waittime}s")
             await asyncio.sleep(waittime)
 
@@ -330,15 +353,19 @@ class eq3bt(FhemModule):
         await utils.run_blocking(fct)
         await self.update_readings()
 
+    def string_to_seconds(self, timestr):
+        ftr = [3600, 60, 1]
+        return sum([a * b for a, b in zip(ftr, map(int, timestr.split(":")))])
+
     # SET Functions BEGIN
-    async def set_on(self, hash):
+    async def set_on(self, hash, params):
         self.create_async_task(
             self.set_and_update(
                 functools.partial(self.thermostat.set_target_temperature, 30)
             )
         )
 
-    async def set_off(self, hash):
+    async def set_off(self, hash, params):
         self.create_async_task(
             self.set_and_update(
                 functools.partial(self.thermostat.set_target_temperature, 4.5)
@@ -353,7 +380,68 @@ class eq3bt(FhemModule):
             )
         )
 
-    async def set_updateStatus(self, hash):
+    async def set_ecoTemperature(self, hash, params):
+        comfort_temp = float(
+            await fhem.ReadingsVal(self.hash["NAME"], "comfortTemperature", "17.0")
+        )
+        self.create_async_task(
+            self.set_and_update(
+                functools.partial(
+                    self.thermostat.set_temperature_presets,
+                    comfort_temp,
+                    params["temp"],
+                )
+            )
+        )
+
+    async def set_comfortTemperature(self, hash, params):
+        eco_temp = float(
+            await fhem.ReadingsVal(self.hash["NAME"], "ecoTemperature", "15.0")
+        )
+        self.create_async_task(
+            self.set_and_update(
+                functools.partial(
+                    self.thermostat.set_temperature_presets, params["temp"], eco_temp
+                )
+            )
+        )
+
+    async def set_temperatureOffset(self, hash, params):
+        self.create_async_task(
+            self.set_and_update(
+                functools.partial(
+                    self.thermostat.set_temperature_offset, params["offset"]
+                )
+            )
+        )
+
+    async def set_windowOpenTemperature(self, hash, params):
+        duration = await fhem.ReadingsVal(
+            self.hash["NAME"], "windowOpenTime", "0:15:00"
+        )
+        duration_sec = self.string_to_seconds(duration)
+        self.create_async_task(
+            self.set_and_update(
+                functools.partial(
+                    self.thermostat.set_windows_open_config,
+                    params["temp"],
+                    duration_sec,
+                )
+            )
+        )
+
+    async def set_windowOpenTime(self, hash, params):
+        temp = await fhem.ReadingsVal(self.hash["NAME"], "windowOpenTemperature", "5")
+        temp = float(temp)
+        self.create_async_task(
+            self.set_and_update(
+                functools.partial(
+                    self.thermostat.set_windows_open_config, temp, params["seconds"]
+                )
+            )
+        )
+
+    async def set_updateStatus(self, hash, params):
         self.create_async_task(self.update_all())
 
     async def set_boost(self, hash, params):
@@ -377,12 +465,12 @@ class eq3bt(FhemModule):
             )
         )
 
-    async def set_eco(self, hash):
+    async def set_eco(self, hash, params):
         self.create_async_task(
             self.set_and_update(functools.partial(self.thermostat.activate_eco))
         )
 
-    async def set_comfort(self, hash):
+    async def set_comfort(self, hash, params):
         self.create_async_task(
             self.set_and_update(functools.partial(self.thermostat.activate_comfort))
         )
@@ -414,6 +502,15 @@ class FhemThermostat(eq3.Thermostat):
         for day in range(0, 6):
             super().query_schedule(day)
 
+    def set_temperature_presets(self, comfort_temp, eco_temp):
+        self.temperature_presets(comfort_temp, eco_temp)
+
+    def set_temperature_offset(self, temp):
+        self.temperature_offset = temp
+
+    def set_windows_open_config(self, temperature, duration):
+        self.window_open_config(temperature, duration)
+
     def set_target_temperature(self, temp):
         self.target_temperature = temp
 
