@@ -6,6 +6,7 @@ import json
 import logging
 import site
 import sys
+import os
 import time
 import traceback
 
@@ -48,9 +49,11 @@ async def activate_internal_modules():
 
 
 async def pybinding(websocket, path):
+    global zc_info
     if zc_info is not None:
         # FHEM discovered us, stop zeroconf
         await zeroconf.get_instance(logger).unregister_service(zc_info)
+        zc_info = None
         zeroconf.get_instance(logger).stop()
 
     global connection_start
@@ -165,7 +168,7 @@ class PyBinding:
                         "update finished...please wait",
                         1,
                     )
-                    sys.exit(1)
+                    os._exit(1)
                 if hash["msgtype"] == "function":
                     # this is needed to avoid 2 replies on dep installation
                     fhem_reply_done = False
@@ -404,6 +407,7 @@ def usage():
 
 
 def run():
+    logging.getLogger("asyncio").setLevel(logging.WARNING)
     try:
         opts, args = getopt.getopt(
             sys.argv[1:],
