@@ -5,6 +5,7 @@ import json
 import re
 
 import tinytuya
+from tinytuya import wizard as tt_wizard
 
 from .. import fhem, utils
 from ..generic import FhemModule
@@ -64,7 +65,7 @@ class tuya(FhemModule):
             # set internal
             hash["DEVICEID"] = self.tt_did
             # set attributes
-            attr_config = {
+            self.attr_config = {
                 "interval": {
                     "default": 15,
                     "format": "int",
@@ -78,9 +79,9 @@ class tuya(FhemModule):
                     "default": "off",
                 },
             }
-            self.set_attr_config(attr_config)
+            self.set_attr_config(self.attr_config)
             # this is needed to set default values
-            await utils.handle_define_attr(attr_config, self, hash)
+            await utils.handle_define_attr(self.attr_config, self, hash)
             # create device
             await self.create_device()
             self.create_async_task(self.update_loop())
@@ -319,6 +320,11 @@ class tuya(FhemModule):
         # dp_1 = "switch_1"
         # dp_42 = "switch_overcharge"
         if self.local_dev is not None:
+            dev = tinytuya.OutletDevice(self.tt_did, self.tt_ip, self.tt_localkey)
+            dev.set_version(self.tt_version)
+            self.local_dev["dps"] = await utils.run_blocking(
+                functools.partial(dev.status)
+            )
             dps = []
             for dp in self.local_dev["dps"]["dps"]:
                 dps.append(f"dp_{int(dp):02d}")
@@ -333,9 +339,9 @@ class tuya(FhemModule):
                     "default": "",
                     "options": ",".join(options),
                 }
-            self._conf_attr.update(attr_conf)
-            self.set_attr_config(self._conf_attr)
-            await utils.handle_define_attr(attr_conf, self, self.hash)
+            self.attr_config.update(attr_conf)
+            self.set_attr_config(self.attr_config)
+            await utils.handle_define_attr(self.attr_config, self, self.hash)
 
         await fhem.readingsSingleUpdate(
             self.hash, "state", "Ready to configure dp_ attributes", 1
@@ -430,7 +436,7 @@ class tuya(FhemModule):
     async def _request_tuyacloud(self, uri, token=None):
         return await utils.run_blocking(
             functools.partial(
-                tinytuya.tuyaPlatform,
+                tt_wizard.tuyaPlatform,
                 self.tt_region,
                 self.tt_key,
                 self.tt_secret,
@@ -535,8 +541,8 @@ class tuya(FhemModule):
             )
 
             # do not create a device if unknown
-            if productid not in mappings.knownSchemas:
-                continue
+            # if productid not in mappings.knownSchemas:
+            #    continue
 
             if not await fhem.checkIfDeviceExists(
                 self.hash, "PYTHONTYPE", "tuya", "DEVICEID", id
